@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2008-2011 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +17,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Authors: William Jon McCann <mccann@jhu.edu>
- *
  */
 
 #include "config.h"
@@ -73,8 +72,6 @@ struct GSLockPlugPrivate
 
         GtkWidget   *notebook;
         GtkWidget   *auth_face_image;
-        GtkWidget   *auth_realname_label;
-        GtkWidget   *auth_username_label;
         GtkWidget   *auth_prompt_label;
         GtkWidget   *auth_prompt_entry;
         GtkWidget   *auth_prompt_box;
@@ -84,7 +81,6 @@ struct GSLockPlugPrivate
 
         GtkWidget   *auth_unlock_button;
         GtkWidget   *auth_switch_button;
-        GtkWidget   *auth_cancel_button;
         GtkWidget   *auth_logout_button;
 
         GtkWidget   *auth_prompt_kbd_layout_indicator;
@@ -730,76 +726,106 @@ image_set_from_pixbuf (GtkImage  *image,
         g_object_unref (pixbuf);
 }
 
-static gboolean
-check_user_file (const gchar *filename,
-                 uid_t        user,
-                 gssize       max_file_size,
-                 gboolean     relax_group,
-                 gboolean     relax_other)
+static GdkPixbuf *
+get_pixbuf_of_user_icon (GSLockPlug *plug)
 {
-        struct stat fileinfo;
+        GError          *error;
+        GDBusConnection *system_bus;
+        GVariant        *find_user_by_name_reply;
+        const char      *user;
+        GVariant        *get_icon_file_reply;
+        GVariant        *icon_file_variant;
+        const char      *icon_file;
+        GdkPixbuf       *pixbuf;
 
-        if (max_file_size < 0) {
-                max_file_size = G_MAXSIZE;
+        error = NULL;
+        system_bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+
+        if (error != NULL) {
+                g_warning ("Unable to get system bus: %s", error->message);
+                g_error_free (error);
+                return NULL;
         }
 
-        /* Exists/Readable? */
-        if (g_stat (filename, &fileinfo) < 0) {
-                return FALSE;
+        find_user_by_name_reply = g_dbus_connection_call_sync (system_bus,
+                                                               "org.freedesktop.Accounts",
+                                                               "/org/freedesktop/Accounts",
+                                                               "org.freedesktop.Accounts",
+                                                               "FindUserByName",
+                                                               g_variant_new ("(s)",
+                                                                              g_get_user_name ()),
+                                                               G_VARIANT_TYPE ("(o)"),
+                                                               G_DBUS_CALL_FLAGS_NONE,
+                                                               -1,
+                                                               NULL,
+                                                               &error);
+        if (error != NULL) {
+                g_warning ("Couldn't find user in accounts service: %s", error->message);
+                g_error_free (error);
+                return NULL;
         }
 
-        /* Is a regular file */
-        if (G_UNLIKELY (!S_ISREG (fileinfo.st_mode))) {
-                return FALSE;
+        user = g_variant_get_string (g_variant_get_child_value (find_user_by_name_reply,
+                                                                0),
+                                     NULL);
+
+        get_icon_file_reply = g_dbus_connection_call_sync (system_bus,
+                                                           "org.freedesktop.Accounts",
+                                                           user,
+                                                           "org.freedesktop.DBus.Properties",
+                                                           "Get",
+                                                           g_variant_new ("(ss)",
+                                                                          "org.freedesktop.Accounts.User",
+                                                                          "IconFile"),
+                                                           G_VARIANT_TYPE ("(v)"),
+                                                           G_DBUS_CALL_FLAGS_NONE,
+                                                           -1,
+                                                           NULL,
+                                                           &error);
+        g_variant_unref (find_user_by_name_reply);
+
+        if (error != NULL) {
+                g_warning ("Couldn't find user icon in accounts service: %s", error->message);
+                g_error_free (error);
+                return NULL;
         }
 
-        /* Owned by user? */
-        if (G_UNLIKELY (fileinfo.st_uid != user)) {
-                return FALSE;
+        g_variant_get_child (get_icon_file_reply, 0, "v", &icon_file_variant);
+
+        icon_file = g_variant_get_string (icon_file_variant, NULL);
+
+        if (icon_file == NULL) {
+                char *string;
+
+                string = g_variant_print (get_icon_file_reply, TRUE);
+                g_warning ("reply for user icon path returned invalid response '%s'", string);
+                g_free (string);
+
+                pixbuf = NULL;
+        } else {
+                pixbuf = gdk_pixbuf_new_from_file_at_size (icon_file,
+                                                           64,
+                                                           64,
+                                                           &error);
+        }
+        g_variant_unref (icon_file_variant);
+        g_variant_unref (get_icon_file_reply);
+
+        if (error != NULL) {
+                g_warning ("Couldn't load user icon: %s", error->message);
+                g_error_free (error);
+                return NULL;
         }
 
-        /* Group not writable or relax_group? */
-        if (G_UNLIKELY ((fileinfo.st_mode & S_IWGRP) == S_IWGRP && !relax_group)) {
-                return FALSE;
-        }
-
-        /* Other not writable or relax_other? */
-        if (G_UNLIKELY ((fileinfo.st_mode & S_IWOTH) == S_IWOTH && !relax_other)) {
-                return FALSE;
-        }
-
-        /* Size is kosher? */
-        if (G_UNLIKELY (fileinfo.st_size > max_file_size)) {
-                return FALSE;
-        }
-
-        return TRUE;
+        return pixbuf;
 }
 
 static gboolean
 set_face_image (GSLockPlug *plug)
 {
         GdkPixbuf    *pixbuf;
-        const char   *homedir;
-        char         *path;
-        int           icon_size = 96;
-        gsize         user_max_file = 65536;
-        uid_t         uid;
 
-        homedir = g_get_home_dir ();
-        uid = getuid ();
-
-        path = g_build_filename (homedir, ".face", NULL);
-
-        pixbuf = NULL;
-        if (check_user_file (path, uid, user_max_file, 0, 0)) {
-                pixbuf = gdk_pixbuf_new_from_file_at_size (path,
-                                                           icon_size,
-                                                           icon_size,
-                                                           NULL);
-        }
-
-        g_free (path);
+        pixbuf = get_pixbuf_of_user_icon (plug);
 
         if (pixbuf == NULL) {
                 return FALSE;
@@ -1201,6 +1227,8 @@ gs_lock_plug_enable_prompt (GSLockPlug *plug,
                             const char *message,
                             gboolean    visible)
 {
+        char *markup;
+
         g_return_if_fail (GS_IS_LOCK_PLUG (plug));
 
         gs_debug ("Setting prompt to: %s", message);
@@ -1208,7 +1236,9 @@ gs_lock_plug_enable_prompt (GSLockPlug *plug,
         gtk_widget_set_sensitive (plug->priv->auth_unlock_button, TRUE);
         gtk_widget_show (plug->priv->auth_unlock_button);
         gtk_widget_grab_default (plug->priv->auth_unlock_button);
-        gtk_label_set_text (GTK_LABEL (plug->priv->auth_prompt_label), message);
+        markup = g_strdup_printf ("<b><big>%s</big></b>", message);
+        gtk_label_set_markup (GTK_LABEL (plug->priv->auth_prompt_label), markup);
+        g_free (markup);
         gtk_widget_show (plug->priv->auth_prompt_label);
         gtk_entry_set_visibility (GTK_ENTRY (plug->priv->auth_prompt_entry), visible);
         gtk_widget_set_sensitive (plug->priv->auth_prompt_entry, TRUE);
@@ -1236,8 +1266,6 @@ gs_lock_plug_disable_prompt (GSLockPlug *plug)
         gtk_widget_set_sensitive (plug->priv->auth_unlock_button, FALSE);
         gtk_widget_set_sensitive (plug->priv->auth_prompt_entry, FALSE);
         /* gtk_widget_hide (plug->priv->auth_unlock_button); */
-
-        gtk_widget_grab_default (plug->priv->auth_cancel_button);
 }
 
 void
@@ -1311,43 +1339,6 @@ gs_lock_plug_add_button (GSLockPlug  *plug,
         return button;
 }
 
-static char *
-get_user_display_name (void)
-{
-        const char *name;
-        char       *utf8_name;
-
-        name = g_get_real_name ();
-
-        if (name == NULL || strcmp (name, "Unknown") == 0) {
-                name = g_get_user_name ();
-        }
-
-        utf8_name = NULL;
-
-        if (name != NULL) {
-                utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
-        }
-
-        return utf8_name;
-}
-
-static char *
-get_user_name (void)
-{
-        const char *name;
-        char       *utf8_name;
-
-        name = g_get_user_name ();
-
-        utf8_name = NULL;
-        if (name != NULL) {
-                utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
-        }
-
-        return utf8_name;
-}
-
 static void
 create_page_one_buttons (GSLockPlug *plug)
 {
@@ -1369,10 +1360,6 @@ create_page_one_buttons (GSLockPlug *plug)
         gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_logout_button), FALSE);
         gtk_widget_set_no_show_all (plug->priv->auth_logout_button, TRUE);
 
-        plug->priv->auth_cancel_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
-                                                                   plug->priv->auth_action_area,
-                                                                   GTK_STOCK_CANCEL);
-        gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_cancel_button), FALSE);
 
         plug->priv->auth_unlock_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
                                                                    plug->priv->auth_action_area,
@@ -1384,130 +1371,6 @@ create_page_one_buttons (GSLockPlug *plug)
         gs_profile_end ("page one buttons");
 }
 
-/* adapted from GDM */
-static char *
-expand_string (const char *text)
-{
-        GString       *str;
-        const char    *p;
-        char          *username;
-        int            i;
-        int            n_chars;
-        struct utsname name;
-
-        str = g_string_sized_new (strlen (text));
-
-        p = text;
-        n_chars = g_utf8_strlen (text, -1);
-        i = 0;
-
-        while (i < n_chars) {
-                gunichar ch;
-
-                ch = g_utf8_get_char (p);
-
-                /* Backslash commands */
-                if (ch == '\\') {
-                        p = g_utf8_next_char (p);
-                        i++;
-                        ch = g_utf8_get_char (p);
-
-                        if (i >= n_chars || ch == '\0') {
-                                g_warning ("Unescaped \\ at end of text\n");
-                                goto bail;
-                        } else if (ch == 'n') {
-                                g_string_append_unichar (str, '\n');
-                        } else {
-                                g_string_append_unichar (str, ch);
-                        }
-                } else if (ch == '%') {
-                        p = g_utf8_next_char (p);
-                        i++;
-                        ch = g_utf8_get_char (p);
-
-                        if (i >= n_chars || ch == '\0') {
-                                g_warning ("Unescaped %% at end of text\n");
-                                goto bail;
-                        }
-
-                        switch (ch) {
-                        case '%':
-                                g_string_append (str, "%");
-                                break;
-                        case 'c':
-                                /* clock */
-                                break;
-                        case 'd':
-                                /* display */
-                                g_string_append (str, g_getenv ("DISPLAY"));
-                                break;
-                        case 'h':
-                                /* hostname */
-                                g_string_append (str, g_get_host_name ());
-                                break;
-                        case 'm':
-                                /* machine name */
-                                uname (&name);
-                                g_string_append (str, name.machine);
-                                break;
-                        case 'n':
-                                /* nodename */
-                                uname (&name);
-                                g_string_append (str, name.nodename);
-                                break;
-                        case 'r':
-                                /* release */
-                                uname (&name);
-                                g_string_append (str, name.release);
-                                break;
-                        case 'R':
-                                /* Real name */
-                                username = get_user_display_name ();
-                                g_string_append (str, username);
-                                g_free (username);
-                                break;
-                        case 's':
-                                /* system name */
-                                uname (&name);
-                                g_string_append (str, name.sysname);
-                                break;
-                        case 'U':
-                                /* Username */
-                                username = get_user_name ();
-                                g_string_append (str, username);
-                                g_free (username);
-                                break;
-                        default:
-                                if (ch < 127) {
-                                        g_warning ("unknown escape code %%%c in text\n", (char)ch);
-                                } else {
-                                        g_warning ("unknown escape code %%(U%x) in text\n", (int)ch);
-                                }
-                        }
-                } else {
-                        g_string_append_unichar (str, ch);
-                }
-                p = g_utf8_next_char (p);
-                i++;
-        }
-
- bail:
-
-        return g_string_free (str, FALSE);
-}
-
-static void
-expand_string_for_label (GtkWidget *label)
-{
-        const char *template;
-        char       *str;
-
-        template = gtk_label_get_label (GTK_LABEL (label));
-        str = expand_string (template);
-        gtk_label_set_label (GTK_LABEL (label), str);
-        g_free (str);
-}
-
 static void
 create_page_one (GSLockPlug *plug)
 {
@@ -1515,7 +1378,6 @@ create_page_one (GSLockPlug *plug)
         GtkWidget            *vbox;
         GtkWidget            *vbox2;
         GtkWidget            *hbox;
-        char                 *str;
 
         gs_profile_start ("page one");
 
@@ -1525,42 +1387,23 @@ create_page_one (GSLockPlug *plug)
         vbox = gtk_vbox_new (FALSE, 12);
         gtk_container_add (GTK_CONTAINER (align), vbox);
 
-        plug->priv->auth_face_image = gtk_image_new ();
-        gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_face_image, TRUE, TRUE, 0);
-        gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 0.5, 1.0);
-
-        vbox2 = gtk_vbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
-
-        str = g_strdup ("<span size=\"x-large\">%R</span>");
-        plug->priv->auth_realname_label = gtk_label_new (str);
-        g_free (str);
-        expand_string_for_label (plug->priv->auth_realname_label);
-        gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_realname_label), 0.5, 0.5);
-        gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_realname_label), TRUE);
-        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_realname_label, FALSE, FALSE, 0);
-
-        /* To translators: This expands to USERNAME on HOSTNAME */
-        str = g_strdup_printf ("<span size=\"small\">%s</span>", _("%U on %h"));
-        plug->priv->auth_username_label = gtk_label_new (str);
-        g_free (str);
-        expand_string_for_label (plug->priv->auth_username_label);
-        gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_username_label), 0.5, 0.5);
-        gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_username_label), TRUE);
-        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_username_label, FALSE, FALSE, 0);
-
-        vbox2 = gtk_vbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
-
         hbox = gtk_hbox_new (FALSE, 6);
-        gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+
+        plug->priv->auth_face_image = gtk_image_new ();
+        gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_face_image, FALSE, FALSE, 0);
+        gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 0, 0);
+
+        vbox2 = gtk_vbox_new (FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
+        gtk_container_set_border_width (GTK_CONTAINER (vbox2), 10);
 
         plug->priv->auth_prompt_label = gtk_label_new_with_mnemonic (_("_Password:"));
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_prompt_label), 0, 0.5);
-        gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_prompt_label, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_prompt_label, FALSE, FALSE, 0);
 
         plug->priv->auth_prompt_entry = gtk_entry_new ();
-        gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_prompt_entry, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_prompt_entry, TRUE, TRUE, 0);
 
         gtk_label_set_mnemonic_widget (GTK_LABEL (plug->priv->auth_prompt_label),
                                        plug->priv->auth_prompt_entry);
@@ -1596,13 +1439,6 @@ unlock_button_clicked (GtkButton  *button,
         gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_OK);
 }
 
-static void
-cancel_button_clicked (GtkButton  *button,
-                       GSLockPlug *plug)
-{
-        gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_CANCEL);
-}
-
 static void
 switch_user_button_clicked (GtkButton  *button,
                             GSLockPlug *plug)
@@ -1695,14 +1531,6 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         gtk_widget_grab_default (plug->priv->auth_unlock_button);
 
-        if (plug->priv->auth_username_label != NULL) {
-                expand_string_for_label (plug->priv->auth_username_label);
-        }
-
-        if (plug->priv->auth_realname_label != NULL) {
-                expand_string_for_label (plug->priv->auth_realname_label);
-        }
-
         if (! plug->priv->logout_enabled || ! plug->priv->logout_command) {
                 if (plug->priv->auth_logout_button != NULL) {
                         gtk_widget_hide (plug->priv->auth_logout_button);
@@ -1723,9 +1551,6 @@ gs_lock_plug_init (GSLockPlug *plug)
         g_signal_connect (plug->priv->auth_unlock_button, "clicked",
                           G_CALLBACK (unlock_button_clicked), plug);
 
-        g_signal_connect (plug->priv->auth_cancel_button, "clicked",
-                          G_CALLBACK (cancel_button_clicked), plug);
-
         if (plug->priv->status_message_label) {
                 if (plug->priv->status_message) {
                         gtk_label_set_text (GTK_LABEL (plug->priv->status_message_label),
@@ -1748,6 +1573,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         g_signal_connect (plug, "delete_event", G_CALLBACK (delete_handler), NULL);
 
+        gtk_widget_set_size_request (GTK_WIDGET (plug), 450, -1);
+
         gs_profile_end (NULL);
 }
 
